local Snek_3d_Sphere = require "snek_3d.snek_3d_sphere"
local Snek_3d_Polygon = require "snek_3d.snek_3d_polygon"
local Snek_3d = {}
Snek_3d.__index = Snek_3d

function Snek_3d:create (objects)
	local o = {}
	for i = 1, #objects do
		local type = objects[i][1]
		if objects[i][1] == "sphere" then
			table.remove(objects[i], 1)
			o[#o + 1] = Snek_3d_Sphere:create(unpack(objects[i]))
		elseif objects[i][1] == "poly" then
			table.remove(objects[i], 1)
			o[#o + 1] = Snek_3d_Polygon:create(unpack(objects[i]))
		end
	end
	local snek = {
		objects = o,
		rotation_x = 0,
		rotation_y = 0,
		rotation_z = 0,
	}
	setmetatable(snek, Snek_3d)
	return snek
end

function Snek_3d:update ()
	for i = 1, #self.objects do
		self.objects[i]:update()
	end
	self:apply_rotation ()
end

function Snek_3d:draw ()
	local sorted_objects = {unpack(self.objects)}
	table.sort(
		sorted_objects,
		function (left, right)
			local vertices_left_z = 0
			for i = 1, #left.vertices do
				vertices_left_z = vertices_left_z + left.vertices[i].screen_z
			end
			vertices_left_z = vertices_left_z / #left.vertices

			local vertices_right_z = 0
			for i = 1, #right.vertices do
				vertices_right_z = vertices_right_z + right.vertices[i].screen_z
			end
			vertices_right_z = vertices_right_z / #right.vertices

			return vertices_left_z > vertices_right_z
		end
	)
	for i = 1, #sorted_objects do
		sorted_objects[i]:draw()
	end
end

function Snek_3d:add_object (object)
	self.objects[#self.objects + 1] = object
end

function Snek_3d:remove_object (object)
	for i = 1, #self.objects do
		if self.objects[i] == object then
			table.remove(self.objects, i)
			return
		end
	end
end

function Snek_3d:apply_rotation ()
	local sin_x = math.sin(self.rotation_x);
	local cos_x = math.cos(self.rotation_x);

	local sin_z = math.sin(self.rotation_z);
	local cos_z = math.cos(self.rotation_z);

	local sin_y = math.sin(self.rotation_y);
	local cos_y = math.cos(self.rotation_y);

	for i = 1, #self.objects do
		for j = 1, #self.objects[i].vertices do
			local vertex = self.objects[i].vertices[j]
			local orig_x = vertex.screen_x
			local orig_z = vertex.screen_z
			vertex.screen_x = (orig_x * cos_y - orig_z * sin_y)
			vertex.screen_z = (orig_z * cos_y + orig_x * sin_y)

			local screen_x = vertex.screen_x
			local screen_y = vertex.screen_y
			vertex.screen_x = (screen_x * cos_z - screen_y * sin_z)
			vertex.screen_y = (screen_y * cos_z + screen_x * sin_z)

			local screen_z = vertex.screen_z
			local screen_y = vertex.screen_y
			vertex.screen_z = (screen_z * cos_x - screen_y * sin_x)
			vertex.screen_y = (screen_y * cos_x + screen_z * sin_x)
		end
	end
end

return Snek_3d
