local LG = love.graphics
local Snek3d_Vertex = require "snek_3d.snek_3d_vertex"

local Snek_3d_Polygon = {
	type = "poly"
}
Snek_3d_Polygon.__index = Snek_3d_Polygon

function Snek_3d_Polygon:create (coords, options)
	local options = options or {}
	local offset = options.offset or {0, 0, 0}
	local v = {}
	for i = 1, #coords, 3 do
		v[#v + 1] = Snek3d_Vertex:create(coords[i], coords[i + 1], coords[i + 2], offset[1], offset[2], offset[3])
	end

	local line = {
		vertices = v,
		stroke_color = options.stroke_color or nil,
		fill_color = options.fill_color or nil,
		stroke_width = options.stroke_width or nil,
	}
	setmetatable(line, Snek_3d_Polygon)
	line:update()
	return line
end

	function Snek_3d_Polygon:update ()
		for i = 1, #self.vertices do
		self.vertices[i]:apply_rotation()
	end
end

function Snek_3d_Polygon:draw ()
	local screen_coords = {}
	for i = 1, #self.vertices do
		screen_coords[#screen_coords + 1] = self.vertices[i].screen_x
		screen_coords[#screen_coords + 1] = self.vertices[i].screen_y
	end
	if self.fill_color and #screen_coords > 4 then
		LG.setColor(self.fill_color)
		LG.polygon("fill", unpack(screen_coords))
	end
	if self.stroke_color and self.stroke_width then
		LG.setLineJoin("none")
		LG.setColor(self.stroke_color)
		LG.setLineWidth(self.stroke_width)
		LG.line(unpack(screen_coords))
		for i = 1, #screen_coords - 1, 2 do
			LG.circle("fill", screen_coords[i], screen_coords[i + 1], self.stroke_width / 2)
		end
	end
end

return Snek_3d_Polygon
